<?php

namespace Service;

use Util\InstanceProvider;
use Util\Messages;
use Zeedhi\Framework\Controller\Exception;
use Util\Utilities;
use Zeedhi\Framework\Session\Session;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class Resource
{
    const ACL_HIDE = 'I';
    const ACL_READ_ONLY = 'R';
    const ACL_READ_WRITE = 'A';

    const WORK_RESOURCE = Entities\Resource::WORK_RESOURCE;
    const ASSET_RESOURCE = Entities\Resource::ASSET_RESOURCE;
    const COST_RESOURCE = Entities\Resource::COST_RESOURCE;
    const CONTACT_RESOURCE = Entities\Resource::CONTACT_RESOURCE;

    const CLASS_SPONSOR = Entities\Resource::CLASS_SPONSOR;
    const CLASS_EMPLOYEE = Entities\Resource::CLASS_EMPLOYEE;
    const CLASS_CONTACT = Entities\Resource::CLASS_CONTACT;
    const CLASS_ASSET = Entities\Resource::CLASS_ASSET;
    const CLASS_COST = Entities\Resource::CLASS_COST;

    const IS_USER_TRUE = Entities\Resource::IS_USER_TRUE;
    const IS_USER_FALSE = Entities\Resource::IS_USER_FALSE;

    const IS_ACTIVE = Entities\Resource::IS_ACTIVE;
    const IS_INACTIVE = Entities\Resource::IS_INACTIVE;
    const IS_DELETED = Entities\Resource::IS_DELETED;

    private static $RESOURCES_TYPE = array(
        self::WORK_RESOURCE => 'Trabalho',
        self::ASSET_RESOURCE => 'Material',
        self::COST_RESOURCE => 'Custo',
        self::CONTACT_RESOURCE => 'Contato');

    protected $entityManager;
    protected $environment;
    protected $calendarService;
    protected $permissionService;
    protected $resourceFactory;
    protected $costResourceService;
    protected $availableService;
    protected $partnerProfileFactory;
    /** @var Structure */
    protected $structureService;
    /** @var Repositories\Resource $resourceRepository */
    protected $resourceRepository;

    protected $sessionManager;

    /** @var  Task $taskService */
    protected $taskService;
    /** @var Repositories\ProjectApproval $projectApprovalRepository */
    protected $projectApprovalRepository;
    /** @var Repositories\ResourceProject $projectApprovalRepository */
    protected $resourceProjectRepository;
    /** @var Repositories\City $projectApprovalRepository */
    protected $cityRepository;
    /**
     * @var InitService
     */
    private $initService;
    /**
     * @var Calendar
     */
    private $serviceCalendar;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Util\Utilities $environment, Calendar $calendarService,
                                Permission $permissionService, \Factory\Resource $resourceFactory, CostResource $costResourceService,
                                Available $availableService, \Factory\PartnerProfile $partnerProfileFactory, Structure $structureService,
                                Session $sessionManager, InitService $initService,Calendar $serviceCalendar,$attachmentsBaseDirectory)
    {

        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->calendarService = $calendarService;
        $this->permissionService = $permissionService;
        $this->resourceFactory = $resourceFactory;
        $this->costResourceService = $costResourceService;
        $this->availableService = $availableService;
        $this->partnerProfileFactory = $partnerProfileFactory;
        $this->structureService = $structureService;
        $this->sessionManager = $sessionManager;
        $this->resourceRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Resource');
        $this->projectApprovalRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'ProjectApproval');
        $this->resourceProjectRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR .  'ResourceProject');
        $this->cityRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR .  'City');
        $this->initService = $initService;
        $this->serviceCalendar = $serviceCalendar;
        $this->profileBaseDirectory = $attachmentsBaseDirectory . DIRECTORY_SEPARATOR . 'profile';
        \Util\InstanceProvider::getLog()->setLog('instanciando Resource service',false,2);
    }

    public static function getNameTypeResourceByControl($type)
    {
        return self::$RESOURCES_TYPE[$type];
    }

    /**
     * Gets the list of resources from database
     * resources screen.
     * @param array $filter
     * @return array
     */
    public function getResources($filter = array())
    {
        $managers = array();
        $allManagers = false;
        $keyParamResponsible = false;
        $isResponsibleFilter = false;
        $isStructureFilter = false;
        if($filter) {
            foreach ($filter as $key => $param) {
                if ($param['columnName'] === 'RESPONSIBLE') {
                    foreach ($param['value'] as $value) {
                        if (!is_null($value)) {
                            array_push($managers, $value);
                        }
                    }
                    $keyParamResponsible = $key;
                    $isResponsibleFilter = true;
                } else if ($param['columnName'] === 'STRUCTURE') {
                    $isStructureFilter = true;
                }

            }
        }
        if ($keyParamResponsible !== false) {
            unset($filter[$keyParamResponsible]);
        }

        if (empty($managers)) {
            $allManagers = true;
        }
        
        $structuresOwnedByLoggedUser = $this->structureService->getStructuresOwnedByManagerOnline($managers, $allManagers);
        $structures = '';
        
        if(!empty($structuresOwnedByLoggedUser)){
            $structures = array();
            foreach($structuresOwnedByLoggedUser as $structureOwnedByLoggedUser){
                $structures[] = $structureOwnedByLoggedUser['ID'];
            }
            $structures = implode(",", $structures);
        }
        
        $internalConditions = "";
        $externalConditions = "";
        
        $resourcesType = Utilities::getValueInFilterDataSourceByName($filter, 'RESOURCES_TYPE', true);
        if ($resourcesType){
            $internalConditions .= "AND R.RESOURCES_TYPE IN ('". implode("','", $resourcesType) ."')";
        }
        
        if(!empty($structures) && ($isResponsibleFilter || $isStructureFilter)){
			$internalConditions .= "AND PER.STRUCTURE IN ( {$structures} ) ";
		}
		
		$structure = Utilities::getValueInFilterDataSourceByName($filter, 'STRUCTURE', true);
		if ($structure){
		    $internalConditions .= "AND PER.STRUCTURE IN (". implode(",", $structure) .")";
		}
	    $externalConditions = Utilities::createQueryFiltered($filter);
		
        $viewableResources = $this->resourceRepository->getResources($this->environment->getCurrentOrganizationId(), $internalConditions, $externalConditions);

        foreach ($viewableResources as &$resource) {
            $resource['TYPE'] = self::getNameTypeResourceByControl($resource['RESOURCES_TYPE']);
        }
        return $viewableResources;
    }

    /**
     * Returns a Resource Entity by its id.
     *
     * @param $id
     * @return null|object
     */
    public function getResourceEntityById($id){
        return $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Resource')->find($id);
    }

    public function getEnvironment()
    {
        return $this->environment;
    }

    public function createResource($externalId, $type, $isUser, $status, $login, $password, $name, $description,
                                   $class = null, $nickName = null, $documentNumber = null, $phone = null,
                                   $address = null, $district = null, $city = null, $zipCode = null, $parent = null,
                                   $availableFrom = null, $rating = 1)
    {
        $entity = $this->resourceFactory->createResource($externalId, $type, $isUser, $status, $login, $password, $name, $description, $class, $nickName,
            $documentNumber, $phone, $address, $district, $city, $zipCode, $parent,$availableFrom, $rating);
        $this->entityManager->persist($entity);
        return $entity;
    }

    public function updateResource($resource, $type, $isUser, $status, $login, $name, $description, $class = null, $nickName = null,
                                   $documentNumber = null, $phone = null, $address = null, $district = null, $city = null, $zipCode = null,
                                   $parent = null, $creator = null, $availableFrom = null, $rating = 1)
    {

        $resource = $this->resourceRepository->find($resource);
        if ($city) {
            $city = $this->cityRepository->find($city);
        }
        $resource = $this->resourceFactory->updateResource($resource, $type, $isUser, $status, $login, $name, $description, $class, $nickName, $documentNumber,
            $phone, $address, $district, $city, $zipCode, $parent, $creator,$availableFrom, $rating);

        $this->entityManager->persist($resource);
        return $resource;
    }

    /**
     * @param $externalId
     * @param $userProfiles
     * @param $isInsert
     * @return array
     */
    public function getDataSetIdentityResourceProfile($externalId, $userProfiles, $isInsert)
    {
        $dataSet = array();
        foreach ($userProfiles as $profile) {
            $userProfileId = $isInsert ? '' : $profile['ID'];
            $profileId = $isInsert ? $profile : $profile['PROFILE_ID'];
            $dataSet[] = $this->getDataIdentityUserProfile($externalId, $userProfileId, $profileId, $isInsert);
        }
        $data = array(
            "requestType" => "DataSet",
            "dataset" => $dataSet
        );

        return $data;
    }

    /**
     * @param $externalId
     * @return array
     */
    public static function getIdentityUserProfileFilterByUserId($externalId)
    {
        $userProfileFilter = array(
            "requestType" => "FilterData",
            "filter" => array(array(
                "name" => "USER_ID",
                "operator" => "=",
                "value" => $externalId
            ))
        );
        return $userProfileFilter;
    }
     /**
     * @param $email
     * @return array
     */
    public static function getIdentityUserProfileFilterByEmail($email)
    {
        $userProfileFilter = array(
            "requestType" => "FilterData",
            "filter" => array(array(
                "name" => "EMAIL",
                "operator" => "=",
                "value" => $email
            ))
        );
        return $userProfileFilter;
    }

    /**
     * @param $profiles
     * @param $userProfileData
     * @return array
     */
    public function getManagementIdentityUserProfiles($profiles, $userProfileData)
    {
        $userProfilesToDelete = array();
        $userProfilesToInsert = array();
        $userProfileIds = array();
        foreach ($userProfileData as $userProfile) {
            $userProfileIds[] = $userProfile['PROFILE_ID'];
        }
        foreach ($userProfileIds as $userProfileId) {
            if (!in_array($userProfileId, $profiles)) {
                $userProfilesToDelete[] = $userProfileId;
            }
        }
        foreach ($profiles as $profile) {
            if (!in_array($profile, $userProfileIds)) {
                $userProfilesToInsert[] = $profile;
            }
        }
        return array($userProfilesToInsert, $userProfilesToDelete);
    }

    /**
     * @param $userProfileData
     * @param $userProfilesToDelete
     * @return array
     */
    public function getUserProfileObjectsToDelete($userProfileData, $userProfilesToDelete)
    {
        $userProfilesToDeleteObj = array();
        foreach ($userProfileData as $userProfile) {
            if (in_array($userProfile['PROFILE_ID'], $userProfilesToDelete)) {
                $userProfilesToDeleteObj[] = $userProfile;
            }
        }

        return $userProfilesToDeleteObj;
    }

    /**
     * @param $externalId
     * @param $userProfileId
     * @param $profileId
     * @param $isNew
     * @return array
     */
    public function getDataIdentityUserProfile($externalId, $userProfileId, $profileId, $isNew)
    {
        $data = array(
            "ID" => $userProfileId,
            "USER_ID" => $externalId,
            "PROFILE_ID" => $profileId,
            "ORGANIZATION_ID" => $this->environment->getCurrentExternalOrganizationId(),
            "__is_new" => $isNew
        );

        return $data;
    }

    protected function persistCostForResource($defaultTax, $hourCost, $extraHourTax, $effectiveDate, $allDays, $resource)
    {
        return $this->costResourceService->createAllResourceCosts($effectiveDate, $defaultTax, $extraHourTax, $hourCost, $allDays, $resource);
    }

    protected function persistAvailabilityForResource($since, $until, $resource)
    {
        return $this->availableService->createAvailable($since, $until, $resource);
    }

    public function getExternalUserId($id)
    {
        return $this->resourceRepository->getExternalUserId($id);
    }

    public function getCurrentUserExternalId()
    {
        $currentUserId = $this->environment->getCurrentUserId();
        return $this->resourceRepository->getExternalUserId($currentUserId);
    }

    public function getResourceByExternalId($externalId)
    {
        return $this->resourceRepository->findOneBy(array('EXTERNAL_ID' => $externalId));
    }

    public function persistResource($id, $externalId, $type, $status, $email, $password, $name, $permission,
                                    $description, $structure, $assetType, $profiles,$availableFrom = null, $rating = null)
    {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            if ($type === self::WORK_RESOURCE) {
                $isUser = self::IS_USER_TRUE;
                $class = self::CLASS_EMPLOYEE;
            } else {
                $isUser = self::IS_USER_FALSE;
                if ($type === self::ASSET_RESOURCE) {
                    $class = self::CLASS_ASSET;
                } else if ($type === self::COST_RESOURCE) {
                    $class = self::CLASS_COST;
                } else {
                    $class = null;
                }
            }
            if (empty($id) && $externalId) {
                $resource = $this->resourceRepository->findOneBy(array('externalId' => $externalId));
                if(!empty($resource)){
                    $id = $resource->getId();
                }
            }
            if (empty($id)) {
                $resource = $this->createResource($externalId, $type, $isUser, $status, $email, $password, $name,
                    $description, $class, null, null, null, null, null, null, null, null, $availableFrom, $rating);
            } else {
                $resource = $this->updateResource($id, $type, $isUser, $status, $email, $name, $description, $class,
                    null, null, null, null, null, null, null, null, null, $availableFrom, $rating);
            }

            if (!empty($externalId) && !$resource->getExternalId()) {
                $resource->setExternalId($externalId);
                $this->entityManager->persist($resource);
                $this->entityManager->flush();
            }

            $this->setPermission($permission, $structure, $resource, $isUser);
            $this->setResourceProfile($resource, $profiles);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return $resource;
    }

    /**
     * @param $externalId
     * @param $profiles
     */
    public function setIdentityResourceProfile($externalId, $profiles)
    {
        $userProfileFilter = self::getIdentityUserProfileFilterByUserId($externalId);
        $userProfileData = $this->initService->getUserProfileInIdentity($this->environment->getToken(), $userProfileFilter);
        list($userProfilesToInsert, $userProfilesToDelete) = $this->getManagementIdentityUserProfiles($profiles, $userProfileData);

        if (!empty($userProfilesToInsert)) {
            $data = $this->getDataSetIdentityResourceProfile($externalId, $userProfilesToInsert, true);
            $this->initService->insertUserProfileInIdentity($this->environment->getToken(), $data);
        }
        if (!empty($userProfilesToDelete)) {
            $userProfilesToDeleteObj = $this->getUserProfileObjectsToDelete($userProfileData, $userProfilesToDelete);
            $data = $this->getDataSetIdentityResourceProfile($externalId, $userProfilesToDeleteObj, false);
            $this->initService->deleteUserProfileInIdentity($this->environment->getToken(), $data);
        }
    }

    public function setResourceProfile(Entities\Resource $resource, $profileIds)
    {
        $nrparcnegocio = InstanceProvider::getRepository()->getResource()->getNrparcnegocioByCdoperador($resource->getId());
        $userProfiles = DevOpsUtilities::arrayAssociativeToArray(InstanceProvider::getRepository()->getResource()->findAccessProfileOfResource($resource->getId()),'NRPERFILACESSO');
        $perfacessparcFactory = InstanceProvider::getFactory()->getPerfacessparc();
        $perfacessparcRepository= InstanceProvider::getRepository()->getPerfacessparc();
        foreach ($profileIds as $profileId) {
            if (!in_array($profileId,$userProfiles)) {
                $perfacessparc = $perfacessparcFactory->createPerfacessparc($nrparcnegocio,$profileId);
                $this->entityManager->persist($perfacessparc);
                $this->entityManager->flush();
            }

        }
        /** @var \Teknisa\Model\Geral\Entities\Perfacessparc[] $perfacessparcs */
        $perfacessparcs = $perfacessparcRepository->findBy(array('nrparcnegocio' => $nrparcnegocio, 'nrorg' => InstanceProvider::getEnvironment()->getCurrentOrganizationId()));

        foreach ($perfacessparcs as $perfacessparc) {
            if(!in_array($perfacessparc->getNrperfilacesso(), $profileIds)){
                $this->entityManager->remove($perfacessparc);
                $this->entityManager->flush();
            }
        }
    }

    public function getResourceAssetTypes()
    {
        return $this->resourceRepository->getResourceAssetTypes(
            $this->environment->getCurrentOrganizationId()
        );
    }

    public function findAllEmployee($filters)
    {
        $filterString = $this->environment->createQueryFiltered($filters);
        return $this->resourceRepository->getAllEmployee(
            $this->environment->getCurrentOrganizationId(), $filterString
        );
    }

    public function allResourcesOrganization($filterData = array())
    {
        $filterString = $this->environment->createQueryFiltered($filterData);
        return $this->resourceRepository->allResourcesOrganization($this->environment->getCurrentOrganizationId(), $filterString);
    }

    public function ResourcesOrganization($id)
    {
        return $this->resourceRepository->ResourcesOrganization($this->environment->getCurrentOrganizationId(), $id);
    }

    public function findAccessProfile($filters)
    {
        foreach ($filters as &$filter) {
            if($filter['columnName'] == "ORGANIZATION_ID"){
                $filter['columnName'] = "NRORG";
            }
        }
        $filters = DevOpsUtilities::createQueryFiltered($filters);
        return $this->resourceRepository->findAccessProfile($this->environment->getCurrentOrganizationId(),$filters);
    }

    public function findAccessProfileOfResource($resourceId,$filters)
    {
        $filters = DevOpsUtilities::createQueryFiltered($filters);
        return $this->resourceRepository->findAccessProfileOfResource($resourceId,$filters);
    }

    public function removeAccessProfileOfResource($id)
    {
        $perfacessparcRepository = InstanceProvider::getRepository()->getPerfacessparc();
        $organization = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $perfacessparc = $perfacessparcRepository->findOneBy(array('nrorg'=>$organization, 'nrperfacessparc'=>$id));
        $this->entityManager->remove($perfacessparc);
        $this->entityManager->flush();
    }

    public function saveAccessProfileOfResource($resourceId, $nrperfilacesso)
    {
        $nrparcnegocio = InstanceProvider::getRepository()->getResource()->getNrparcnegocioByCdoperador($resourceId);
        $userProfiles = DevOpsUtilities::arrayAssociativeToArray(InstanceProvider::getRepository()->getResource()->findAccessProfileOfResource($resourceId), 'NRPERFILACESSO');
        $perfacessparcFactory = InstanceProvider::getFactory()->getPerfacessparc();
        if (!in_array($nrperfilacesso, $userProfiles)) {
            $perfacessparc = $perfacessparcFactory->createPerfacessparc($nrparcnegocio, $nrperfilacesso);
            $this->entityManager->persist($perfacessparc);
            $this->entityManager->flush();
        }
    }

    public function getFullResourceDataFromId($id)
    {
        $resourceRepository = $this->resourceRepository;
        $dataSet = $resourceRepository->getBasicDataResource($id, $this->environment->getCurrentOrganizationId());
        foreach ($dataSet as &$row) {
            $row['TYPE_NAME'] = self::getNameTypeResourceByControl($row['TYPE_CONTROL']);
            $row['PROFILE_NAME'] = $this->getAclUserProfilesByResourceId($id);
        }
        return $dataSet;
    }

    public function getIdentityUserProfilesByResourceId($resourceId) {
        $externalId = $this->resourceRepository->find($resourceId)->getExternalId();
        $userProfileFilter = self::getIdentityUserProfileFilterByUserId($externalId);
        $userProfileData = $this->initService->getUserProfileInIdentity($this->environment->getToken(), $userProfileFilter);
        $userProfileIds = array();

        foreach ($userProfileData as $userProfile) {
            $userProfileIds[] = intval($userProfile['PROFILE_ID']);
        }

        return $userProfileIds;
    }

    public function getAclUserProfilesByResourceId($resourceId) {
        return DevOpsUtilities::arrayAssociativeToArray($this->resourceRepository->getAclProfileIds($resourceId));
    }

    public function getResourceAvailableFromId($id)
    {
        return $this->resourceRepository
            ->getDataAvailableResourceList($id, $this->environment->getCurrentOrganizationId());
    }

    public function getResourceCostFromId($id)
    {
        return $this->resourceRepository
            ->getDataCostResourceList($id, $this->environment->getCurrentOrganizationId());
    }

    public function getResourceCalendarFromId($id)
    {
        return $this->resourceRepository
            ->getCalendarResource($id, $this->environment->getCurrentOrganizationId());
    }

    public function getResourceExceptionFromId($id)
    {
        return $this->resourceRepository
            ->getExceptionResource($id, $this->environment->getCurrentOrganizationId());
    }

    public function getResourceProfileFromId($id)
    {
        return $this->resourceRepository
            ->getProfileResourceList($id, $this->environment->getCurrentOrganizationId());
    }

    /**
     * Delete a resource
     * @param $id
     * @return bool
     * @throws Exception
     */
    public function deleteResource($id)
    {
        $resource = $this->resourceRepository->find($id);
        $permissionRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Permission')
            ->findOneBy(array('resource' => $resource, 'type' => Permission::IS_MANAGER));
        if (!empty($permissionRepository)) {
            throw new Exception(Messages::ERR_EXCLUDE_RESOURCE_STRUCTURE_MANAGER);
        }
        $resource->setStatus('D');
        $this->entityManager->persist($resource);
        $this->entityManager->flush();
        return true;
    }/**
     * Delete a resource in Identity
     * @return bool
     * @throws Exception
     */
    public function deleteResourceInIdentity($row, $token)
    {
        if(empty($token)){
            $token = $this->environment->getToken();
        }
        $data = $this->setDataUserForIdentity($row);
        $this->initService->removeUserInIdentity($token,$data);
        return true;
    }
    public function setDataUserForIdentity($row)
    {
        $password = md5(empty($row['PASSWORD'])? '':$row['PASSWORD']);
        $data = array(
            "requestType" => "DataSet",
            "dataset" => array(array(
                "NAME"      			=> $row['NAME'],
                "FIRSTNAME" 			=> $row['FIRSTNAME'],
                "LASTNAME"  			=> $row['LASTNAME'],
                "EMAIL"     			=> empty($row['EMAIL'])?'':$row['EMAIL'],
                "PASSWORD"  			=> $password,
                "STATUS"                => $row['STATUS'],
                "ID"        			=> $this->resourceRepository->find($row['ID'])->getExternalId(),
                "ORGANIZATION_ID" => $this->environment->getCurrentOrganizationId(),
                "__is_new" => empty($row['__is_new'])?'':$row['__is_new'],
            ))
        );
        return $data;
    }

    public function getResourceByProject($project, $param = '', $taskClassificationControl = null)
    {
        $filterAllowedAccessProfile = '';
        if($taskClassificationControl){
            $allowedAccessProfileIds = InstanceProvider::getService()->getClassification()->getAllowedAccessProfileFilter($taskClassificationControl);
            if($allowedAccessProfileIds){
                $filterAllowedAccessProfile = $allowedAccessProfileIds;
            }
        }
        $filter = Utilities::createQueryFiltered($param);
        $resources = $this->resourceRepository->getResourceByProject($project, $filter, $filterAllowedAccessProfile);
        foreach ($resources as $key => $resource) {
            $resources[$key]['EXCEPTION'] = array();
        }
        return $resources;
    }
    public function getTaskResourceMeeting($project,$ids)
    {
        if(empty($ids)){
            $resources = $this->resourceRepository
                ->getResourceByProject($project);
        }else{
            $resources = $this->resourceRepository
                ->getTaskResourceMeeting($ids);
        }


        return $resources;
    }

    public function findManagerByOrganization($resourceExternalIds)
    {
        return $this->resourceRepository->findManagerByOrganization($this->environment->getCurrentOrganizationId(), $this->environment->getCurrentUserId(), $resourceExternalIds);
    }

    public function findManagerProjectEdit( $resourceExternalIds)
    {
        return $this->resourceRepository->findManagerProjectEdit($this->environment->getCurrentOrganizationId(), $resourceExternalIds);
    }

    public function findManagerProjectEditFilterByName($resourceExternalIds, $filter)
    {
        return $this->resourceRepository->findManagerProjectEditByName($this->environment->getCurrentOrganizationId(), $resourceExternalIds, $filter);
    }

    public function findManagerProjectEditFilter($param, $resourceExternalIds)
    {
        return $this->resourceRepository->findManagerProjectEditFilter($this->environment->getCurrentOrganizationId(), $param, $resourceExternalIds);
    }

    public function findMyStructuresManagers($resourceExternalIds)
    {
        $organization = $this->environment->getCurrentOrganizationId();
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Structure $structureRepository */
        $structureRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure');
        $structures = $structureRepository->getGroupManager($organization, $this->environment->getCurrentUserId());
        $iDs = $this->getStructuresId($structures);
        return $this->resourceRepository->findStructuresManagers($iDs, $organization, $resourceExternalIds);
    }

    public function getStructuresId($structures)
    {
        $structuresIDs = [];
        foreach ($structures as $structure) {
            array_push($structuresIDs, intval($structure['ID']));
        }
        return $structuresIDs;
    }

    function getDataSetResourcesForProject($projectId)
    {
        return $this->resourceRepository->getDataSetResourcesForProject($this->environment->getCurrentOrganizationId(), $projectId);
    }

    public function findResourceSponsorByOrganization()
    {
        return $repositoryResource = $this->resourceRepository->getProjectByOrganization($this->environment->getCurrentOrganizationId());
    }

    public function removeResourceAvailability($availabilityId)
    {
        $availability = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Available', $availabilityId);
        $this->entityManager->remove($availability);
        $this->entityManager->flush();
    }

    public function removeResourceCost($effectiveDate)
    {
        $effectiveDate = $this->costResourceService->parseDate($effectiveDate);
        $costs = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\CostResource')
            ->findBy(array('effectiveDate' => $effectiveDate));
        foreach ($costs as $cost) {
            $this->entityManager->remove($cost);
            $this->entityManager->flush();
        }
    }

    public function getResourcesAllocatedInTask($taskId)
    {
        $dataSet = $this->resourceRepository
            ->findResourcesByTaskId($taskId);
        return $dataSet;
    }

    public function returnOnlyAllocatedResources($resourcesIDs)
    {
        $dataSet = array();
        $resourceProjectRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\ResourceProject');
        foreach ($resourcesIDs as $id) {
            $resourceProject = $resourceProjectRepository->findOneBy(array('resourceC' => $id));
            if (!empty($resourceProject)) {
                $resource = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource', $id);
                $resource->getName();
                array_push($dataSet, array(
                    'NAME' => $resource->getName()
                ));
            }
        }
        return $dataSet;
    }

    public function persistAvailability($id, $begin, $end, $resource)
    {
        $resource = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource', $resource);
        if (empty($id)) {
            return $this->availableService->createAvailable($begin, $end, $resource);
        } else {
            return $this->availableService->updateAvailable($id, $begin, $end, $resource);
        }
    }

    public function persistCostResource($defaultTax, $hourCost, $extraHourTax, $effectiveDate, $allDays, $resource)
    {
        $resource = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource', $resource);
        if (!empty($defaultTax)) {
            $defaultTax = Utilities::moneyForDouble($defaultTax);
        }
        if (!empty($hourCost)) {
            $hourCost = Utilities::moneyForDouble($hourCost);
        }
        if (!empty($extraHourTax)) {
            $extraHourTax = Utilities::moneyForDouble($extraHourTax);
        }
        return $this->costResourceService->persistResourceCosts($defaultTax, $hourCost, $extraHourTax, $effectiveDate, $allDays, $resource);
    }

    public function persistResourceProfile($resource, $profile)
    {
        /*
         * Phillipe Silva - 18/12/2015
         * Tabela PARTNER_PROFILE não será mais utilizada.
         * O perfil de acesso do recurso dependerá diretamente do Identity.
         */

//        $resource = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource', $resource);
//        $profile = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Profile', $profile);
//        $entity = $this->partnerProfileFactory->createPartnerProfile($resource, $profile);
//        $this->entityManager->persist($entity);
//        $this->entityManager->flush();
    }

    public function removeResourceProfile($profileId)
    {
        /*
         * Phillipe Silva - 18/12/2015
         * Tabela PARTNER_PROFILE não será mais utilizada.
         * O perfil de acesso do recurso dependerá diretamente do Identity.
         */

//        $profile = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\PartnerProfile', $profileId);
//        $this->entityManager->remove($profile);
//        $this->entityManager->flush();
    }


    public function findResourcesByOrganization()
    {
        return $this->resourceRepository->findResourcesByOrganization($this->environment->getCurrentOrganizationId());
    }

    public function findAllStructureManagers($filters)
    {
        $filters = DevOpsUtilities::createQueryFiltered($filters);
        return $this->resourceRepository->getAllStructureManagers($this->environment->getCurrentOrganizationId(),$filters);
    }

    public function updateResourceTypeUser($resource, $email, $password, $status, $name, $photo = null, $passwordReset= null)
    {
        if(!$passwordReset && ! $this->isEmailAvailable($email)){
            throw new \Exception("O e-mail informado já foi cadastrado para outro usuário.");
        }

        $resourceEntityClass = "\\Zeedhi\\DevOps\\Model\\Projectmanagement\\Entities\\Resource";
        if( ! ($resource instanceof $resourceEntityClass)){
            $resource = $this->entityManager->find($resourceEntityClass, $resource);
        }

        $resourceEntity = $this->resourceFactory->updateResourceTypeUser($resource, $email, $password, $status, $name);
        $this->entityManager->persist($resourceEntity);
        $this->entityManager->flush();

        if( ! empty($photo)){
            $this->uploadPicture($photo, $this->environment->getCurrentUserId());
        }

        return $resourceEntity;
    }

    private function isEmailAvailable($email)
    {
        $resourceEntityClass = "\\Zeedhi\\DevOps\\Model\\Projectmanagement\\Entities\\Resource";
        $usersUsingEmail = $this->entityManager->getRepository($resourceEntityClass)->findBy(array(
            "email" => $email,
            "organization" => $this->environment->getCurrentOrganizationId()
        ));

        if(empty($usersUsingEmail)){
            return true;
        }

        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $user */
        foreach($usersUsingEmail as $user){
            if($user->getId() != $this->environment->getCurrentUserId()){
                return false;
            }
        }

        return true;
    }

    private function uploadPicture($photo, $fileName)
    {
        $dir = $this->profileBaseDirectory;
        if( ! file_exists($dir)){
            mkdir($dir);
        }

        $dir = $dir . DIRECTORY_SEPARATOR;
        file_put_contents($dir . $fileName, $photo);
    }

    /**
     * @param $email
     * @param $password
     * @return array|bool
     */
    public function login($email, $password)
    {
        $user = $this->resourceRepository->getAuthentication($email, $password, false);
        if ($user) {
            return $this->refreshSession($user);
        } else {
            return false;
        }
    }

    /**
     * Refreshes the session when the user enters on Project
     * Management. Does not matter if he is already logged
     * by session or loging in that moment.
     *
     * @param null $user
     * @return array
     */
    public function refreshSession($user = null)
    {
        $user = empty($user) ? $this->environment->getCurrentUser() : $user;
        $this->setSession($user->getOrganization(), $user->getId());
        $resultSet = $this->prepareOrganization($user);
        $infoToLocalStorage['taskParameters'] = $this->findTaskOptionsParameters();
        $resultSet['infoToLocalStorage'] = $infoToLocalStorage;
        return $resultSet;
    }
    /**
     * Finds the parameters already registered.
     * @return array $taskOptionParameters
     */
    public function findTaskOptionsParameters(){
        $organizationId = $this->environment->getCurrentOrganizationId();
        $taskOptionsParameters = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Task')->getTaskOptionsParameters($organizationId);
        return $taskOptionsParameters;
    }

    public function getSession($name)
    {
        return $this->sessionManager->get($name);
    }

    public function hasSession($name)
    {
        return $this->sessionManager->has($name);
    }

    public function setSession($organizationId, $userId)
    {
        $this->sessionManager->set('organizationId', $organizationId);
        $this->sessionManager->set('userId', $userId);
    }

    public function setSessionByCdOperador($cdOperador)
    {
        if($cdOperador){
            /** @var Entities\Resource $currentResource */
            $currentResource = $this->resourceRepository->findOneBy(array('externalCode' => $this->environment->getCurrentCdoperador()));
            $this->sessionManager->set('organizationId', $currentResource->getOrganization());
            $this->sessionManager->set('userId', $currentResource->getId());
        }
    }

    public function clearSession()
    {
        $this->sessionManager->set('organizationId', null);
        $this->sessionManager->set('userId', null);
    }

    public function isSessionStarted()
    {
        return $this->sessionManager->isStarted();
    }

    public function destroySession()
    {
        if ($this->sessionManager->isStarted()) {
            $this->sessionManager->destroy();
        }
    }

    public function getUserData()
    {
        $ds = DIRECTORY_SEPARATOR;
        $userData = $this->environment->getCurrentUser();
        $resourceId = $userData->getId();
        $operadorRepository = InstanceProvider::getRepository()->getOperador();
        $clienteRepository = InstanceProvider::getRepository()->getCliente();
        $externalCode = $this->resourceRepository->find($resourceId)->getExternalCode();
        $cliente = null;
        if ($externalCode) {
            $operador = $operadorRepository->findOneBy(array(
                'cdoperador' => $externalCode
            ));
            if ($operador && $operador->getCdCliente()) {
                 $cliente = $this->resourceRepository->findOneBy(array(
                'externalCode' => $operador->getCdCliente()
                ));
            }
        }
        $userData = array(
            "NAME" => $userData->getName(),
            "EMAIL" => $userData->getEmail(),
            "STATUS" => $userData->getStatus(),
            "PASSWORD" => $userData->getPassword(),
            "RESOURCE_ID" => $resourceId,
            "SPONSOR_ID" => $cliente ? $cliente->getId() : null,
            "SPONSOR" => $cliente ? $cliente->getName() : null,
            "CLASS" => $cliente ? $cliente->getClass() : null,
            "STATUS" => $cliente ? $cliente->getStatus() : null,
            "ORGANIZATION" => $cliente ? $cliente->getOrganization() : null
        );
        
        $dir = $this->profileBaseDirectory . $ds;
        if (file_exists($dir . $resourceId)) {
            $photo = file_get_contents($dir . $resourceId);
        } else {
            $photo = false;
        }
        $userData['PHOTO_VIEW'] = $photo === false ? null : $photo;

        $userData = array($userData);

        return $userData;
    }

    public function getIdentityUserData()
    {
        $email = $this->environment->getCurrentUser()->getEmail();
        $userRepository = $this->resourceRepository;

        /**@var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $user */
        $user = $userRepository->findOneBy(array(
            "email" => $email,
            "organization" => $this->environment->getCurrentOrganizationId(),
            "status" => "A"
        ));
        $externalId = $user->getExternalId();
        return $externalId;
    }

    public function changePassword($currentPassword, $newPassword)
    {
        $email = $this->environment->getCurrentUser()->getEmail();
        $userRepository = $this->resourceRepository;
        $user = $userRepository->findOneBy(array(
            "email" => $email,
            "organization" => $this->environment->getCurrentOrganizationId(),
            "password" => (md5($currentPassword)),
            "status" => "A"
        ));
        if ($user) {
            $this->updateResourceTypeUser($user, $user->getEmail(), (md5($newPassword)), $user->getStatus(), $user->getName());
            return true;
        }
        return false;
    }

    /**
     *
     * This function resets a specified resource's password. Is not needed the current
     * password so, technically, only managers can use this option. It is an alternative
     * for "I forgot my password".
     *
     * @param $newPassword
     * @param $resourceId
     * @return boolean
     */
    public function resetResourcePassword($newPassword, $resourceId)
    {

        $email = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource', $resourceId)->getEmail();
        $user = $this->resourceRepository->findOneBy(array(
            "id" => $resourceId,
            "email" => $email,
            "organization" => $this->environment->getCurrentOrganizationId(),
            "status" => "A"
        ));
        if ($user) {
            $this->updateResourceTypeUser($user, $email, (md5($newPassword)), $user->getStatus(), $user->getName(), null, true);
            return true;
        }
        return false;
    }

    public function saveEditedUserDataValidate($name, $email, $photo = null)
    {
        if( ! Utilities::validateNameLogin($name)){
            throw new \Exception("Nome inválido.");
        }

        if( ! Utilities::validateEmail($email)){
            throw new \Exception("Email inválido.");
        }

        $this->saveEditedUserData($name, $email, $photo);
    }

    public function saveEditedUserData($name, $email, $photo = null)
    {
        $login = $this->environment->getCurrentUser()->getEmail();
        $user = $this->resourceRepository->findOneBy(array(
            "email" => $login,
            "organization" => $this->environment->getCurrentOrganizationId(),
            "status" => self::IS_ACTIVE
        ));

        if ($user) {
            return $this->updateResourceTypeUser($user, $email, $user->getPassword(), $user->getStatus(), $name, $photo);
        }
        return false;
    }

    /**
     * @param $user
     * @return array
     */
    public function prepareOrganization($user)
    {
        $organization = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Organization', $user->getOrganization());
        $resultSet = array(
            'name' => $user->getName(),
            'login' => $user->getEmail(),
            'organization' => $organization->getId(),
            'orgName' => $organization->getName(),
            'idBusinessPartner' => $user->getId()
        );
        return $resultSet;
    }

    /**
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $user
     * @return array
     */
    protected function prepareAclProfile($user)
    {
        /**@var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Rule $ruleRepository */
        $ruleRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Rule');
        /**@var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\PartnerProfile $partnerProfiles */
        $partnerProfiles = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\PartnerProfile')
            ->findBy(array('resource' => $user->getId()));
        $profileRules = array();

        if ($partnerProfiles) {
            $profileRules = $ruleRepository->findBy(array('profile' => $partnerProfiles[0]->getProfile()->getId()));

            foreach ($partnerProfiles as $partnerProfile) {
                $profileRules = $this->prepareRules($profileRules, $ruleRepository, $partnerProfile);
            }
        }
        return $profileRules;
    }

    /**
     * @param $profileRules
     * @return array
     */
    protected function prepareAclRules($profileRules)
    {
        $rules = array();
        $elementRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Element');
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Rule[] $profileRules
         */
        foreach ($profileRules as $rule) {
            $element = $elementRepository->find($rule->getElement());
            $duplicatedElement = false;
            foreach ($rules as $ruleKey => $ruleValue) {
                $ruleValueType = $ruleValue['ruleType'];
                if ($this->isElementRule($ruleValue, $element) &&
                    (
                        ($ruleValueType === self::ACL_READ_WRITE) ||
                        ($ruleValueType === self::ACL_READ_ONLY && $rule->getRuleType() !== self::ACL_READ_WRITE) ||
                        ($ruleValueType === self::ACL_HIDE && $rule->getRuleType() === self::ACL_HIDE)
                    )
                ) {
                    $duplicatedElement = true;
                }
            }
            if (!$duplicatedElement) {
                array_push($rules, array(
                    'ruleType' => $rule->getRuleType(),
                    'elementName' => $element->getName(),
                    'elementType' => $element->getElementType()
                ));
            }
        }
        return $rules;
    }

    /**
     * @param $ruleValue
     * @param $element
     * @return bool
     */
    protected function isElementRule($ruleValue, $element)
    {
        return ($ruleValue['elementName'] === $element->getName() && $ruleValue['elementType'] === $element->getElementType());
    }

    /**
     * @param $assetType
     * @param $resource
     */
    protected function setAssetType($assetType, $resource)
    {
        $partnerProfileModel = '\Zeedhi\DevOps\Model\Projectmanagement\Entities\PartnerProfile';
        $assetTypeEntity = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Profile', $assetType);
        if ($resource instanceof $partnerProfileModel) {
            $partnerProfileEntity = $resource;
        } else {
            $partnerProfileEntity = $this->entityManager->getRepository($partnerProfileModel)->findOneBy(array('resource' => $resource));
        }
        if (empty($partnerProfileEntity)) {
            $partnerProfileEntity = $this->partnerProfileFactory->createPartnerProfile($resource, $assetTypeEntity);
        } else {
            $partnerProfileEntity = $this->partnerProfileFactory->updatePartnerProfile($partnerProfileEntity, $resource, $assetTypeEntity);
        }
        $this->entityManager->persist($partnerProfileEntity);
    }

    /**
     * @param $permission
     * @param $structure
     * @param $resource
     * @param $isUser
     */
    protected function setPermission($permission, $structure, $resource, $isUser)
    {
        if ($permission) {
            $permissionEntity = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Permission', $permission);
            if ($permissionEntity->getStructure()->getId() != $structure) {
                $structureEntity = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure', $structure);
                $permissionEntity = $this->permissionService->updatePermission($permissionEntity, $resource, $structureEntity, $permissionEntity->getType());
                $this->entityManager->persist($permissionEntity);
            }
        } else {
            if ($structure && $isUser === self::IS_USER_TRUE) {
                $structureEntity = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Structure', $structure);
                $permissionEntity = $this->permissionService->createPermission($resource, $structureEntity);
                $this->entityManager->persist($permissionEntity);
            }
        }
    }

    public function getSponsors()
    {
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource $sponsorRepository */
        $sponsorRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource');
        return $sponsorRepository->getSponsorByOrganization($this->environment->getCurrentOrganizationId());
    }

    public function getProfileByCurrentUser()
    {
        $resourceId = InstanceProvider::getEnvironment()->getCurrentUserId();
        $profileIds = Utilities::getArrayFromAssociativeArray(InstanceProvider::getRepository()->getResource()->getAclProfileIds($resourceId));
        $this->environment->setCurrentUserProfiles($profileIds);
        return $profileIds;
    }


    public function createSponsor($businessPartner, $status, $name, $nickName, $documentNumber, $description)
    {
        $entityFactory = new \Factory\Sponsor($this->entityManager);
        $entity = $entityFactory->createSponsor($businessPartner, $name, $status, $nickName, $documentNumber, $description);
        $this->entityManager->persist($entity);
        $this->entityManager->flush();
        return $entity;
    }

    public function updateSponsor($entity, $businessPartner, $status, $name, $nickName, $documentNumber, $description)
    {
        $entityFactory = new \Factory\Sponsor($this->entityManager);
        $entity = $entityFactory->updateSponsor($entity, $businessPartner, $name, $status, $nickName, $documentNumber, $description, $this->environment->getCurrentUser());
        $this->entityManager->persist($entity);
        $this->entityManager->flush();
        return $entity;
    }

    public function deleteSponsor($sponsorId)
    {
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource
         */
        $resource = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource', $sponsorId);
        $parent = $resource->getParent();
        $resource->setStatus('D');
        $this->entityManager->persist($resource);
        $this->entityManager->flush();
        return $parent;
    }

    public function basicSponsor($resource)
    {
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource $resourceRepository */
        $resourceRepository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource');
        return $resourceRepository->basicSponsor($resource);
    }

    public function basicSponsorEdit($sponsor)
    {
        $organization = $this->environment->getCurrentOrganizationId();
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Sponsor $repository
         */
        $Repository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Sponsor');
        return $Repository->getBasicSponsorEdit($sponsor, $organization);
    }

    public function advancedSponsor($sponsor)
    {
        $organization = $this->environment->getCurrentOrganizationId();
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Sponsor $repository
         */
        $Repository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Sponsor');
        return $Repository->getAdvancedSponsor($sponsor, $organization);
    }


    public function contactSponsor($sponsorId)
    {
        $organizationId = $this->environment->getCurrentOrganizationId();
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource $repository
         */
        $repository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource');
        return $repository->getContactSponsor($sponsorId, $organizationId);
    }

    public function sponsorProjectData($sponsorId, $conditions)
    {
        $organization = $this->environment->getCurrentOrganizationId();
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource $repository
         */
        $repository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource');
        $allProjects = $repository->getSponsorProjectData($sponsorId, $organization, $conditions);
        $allProjectsIds = array();
        foreach ($allProjects as $project) {
            array_push($allProjectsIds, $project['ID']);
        }
        foreach ($allProjects as &$project) {
            if (!in_array($project['PARENT'], $allProjectsIds)) {
                $project['PARENT'] = null;
            }
        }
        return $allProjects;
    }

    public function sponsorActive()
    {
        $organization = $this->environment->getCurrentOrganizationId();
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource $repository
         */
        $repository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource');
        return $repository->getSponsorActive($organization);
    }

    public function formContact($contact)
    {
        $organization = $this->environment->getCurrentOrganizationId();
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource $repository
         */
        $repository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource');
        return $repository->formContact($contact, $organization);
    }

    public function formContactList($resource, $filters)
    {
        $organization = $this->environment->getCurrentOrganizationId();
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource $repository
         */
        $repository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource');
        return $repository->formContactList($resource, $organization, $filters);
    }

    public function findSponsorByOrganization()
    {
        return $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Sponsor')
            ->getSponsorByOrganization($this->environment->getCurrentOrganizationId());
    }

    public function newSponsor($login, $password, $status, $name, $nickName, $documentNumber, $description)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            if($login){
                $validateEmail = $this->resourceRepository->findOneBy(array('email' => $login));
                if(!empty($validateEmail)){
                    throw new \Exception("Email já cadastrado. Favor informar outro email.");
                }
            }
            $profile = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Profile')
                ->findOneBy(array('organization' => $this->environment->getCurrentOrganizationId(), 'category' => self::CLASS_SPONSOR));
            $validationName = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource')->findOneBy(array('name' => $name, 'class' => self::IS_USER_TRUE));
            if (!empty($validationName)) {
                throw new \Exception("Já existe um cliente com este nome. Insira outro, por favor.");
            } else {
                if (empty($password) || empty($login)) {
                    $isUser = self::IS_USER_FALSE;
                } else {
                    $isUser = self::IS_USER_TRUE;
                }
                $resource = $this->createResource(null, self::WORK_RESOURCE, $isUser, $status, $login, $password, $name, $description, self::CLASS_SPONSOR, $nickName, $documentNumber);

                /*
                 * Phillipe Silva - 18/12/2015
                 * Tabela PARTNER_PROFILE não será mais utilizada.
                 * O perfil de acesso do recurso dependerá diretamente do Identity.
                 */
//                $partnerProfileEntity = $this->partnerProfileFactory->createPartnerProfile($resource, $profile);
//                $this->entityManager->persist($partnerProfileEntity);
                $this->entityManager->flush();
                $connection->commit();
            }
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
        return $resource;
    }

    public function editSponsor($login, $password, $status, $name, $nickName, $documentNumber, $description, $resourceId)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            if($login){
                $sameEmailResource = $this->resourceRepository->findOneBy(array('email' => $login));
                if(!empty($sameEmailResource) && $sameEmailResource->getId() != $resourceId){
                    throw new \Exception("Email já cadastrado. Favor informar outro email.");
                }
            }
            
            if (empty($password) || empty($login)) {
                $isUser = self::IS_USER_FALSE;
            } else {
                $isUser = self::IS_USER_TRUE;
            }
            $resource = $this->entityManager->find(Utilities::ENTITY_DIR . 'Resource', $resourceId);
            $resource = $this->updateResource($resource, self::WORK_RESOURCE, $isUser, $status, $login, $name, $description, self::CLASS_SPONSOR, $nickName, $documentNumber);
            $resource = $this->resourceFactory->updateResourceTypeUser($resource, $login, $password, $status, $name);
            $this->entityManager->persist($resource);
            $this->entityManager->flush();
            $connection->commit();
            return $resource->getId();
            
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }

    }
    public function createContactByProductCall($sponsorId,$name,$externalCode){
        return $this->createContact($sponsorId, null, null, null, null, $name,null,null,null,$externalCode);
    }
    
    public function createNewContact($sponsorId, $address, $city, $district, $email, $name, $phone = null, $status = null, $zipCode = null,$externalCode = null)
    {
        try{
            if($email){
                $isContact = $this->resourceRepository->findOneBy(array('email' => $email));
                if(!empty($isContact)){
                    throw new \Exception("Email já cadastrado. Favor informar outro email.");
                }
            }
            $contact = $this->createContact($sponsorId, $address, $city, $district, $email, $name, $phone, $status, $zipCode, $externalCode);
            return array("ID" => $contact->getId());
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function createContact($sponsorId, $address, $city, $district, $email, $name, $phone = null,
                                  $status = 'A', $zipCode = null,$externalCode = null)
    {


        $sponsor = $this->resourceRepository->find($sponsorId);
        if ($city) {
            $city = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\City', $city);
        }
        $contact = $this->createResource(null,self::CONTACT_RESOURCE, self::IS_USER_FALSE, $status, $email, null,
            $name, null, self::CLASS_CONTACT, null, null, $phone, $address, $district, $city, $zipCode, $sponsor);
        $contact->setExternalCode($externalCode);
        $this->entityManager->persist($contact);
        $this->entityManager->flush();

        return $contact;
    }

    public function updateContact($contactId, $sponsorId, $address, $city, $district, $email, $name, $phone,
                                  $status, $zipCode)
    {
        try{
            
            if ($email) {
                $isContact = $this->resourceRepository->findOneBy(array('email' => $email));
                if(!empty($isContact) && $isContact->getId() != $contactId){
                    throw new \Exception("Email já cadastrado. Favor informar outro email.");
                }
            }
            
            $contact = $this->resourceRepository->find($contactId);
            $sponsor = $this->resourceRepository->find($sponsorId);
            $contact = $this->updateResource($contact, self::CONTACT_RESOURCE, self::IS_USER_FALSE, $status, $email, $name,
                null, self::CLASS_CONTACT, null, null, $phone, $address, $district, $city, $zipCode, $sponsor);
            $this->entityManager->persist($contact);
            $this->entityManager->flush();
    
            return array("ID" => $contact->getId());
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getDataSetForEditProjectResource($project, $filterNames = false, $filterOperator = false, $filterValue = false, $taskClassificationControl = null)
    {
        $filterAllowedAccessProfile = '';
        if($taskClassificationControl){
            $allowedAccessProfileIds = InstanceProvider::getService()->getClassification()->getAllowedAccessProfileFilter($taskClassificationControl);
            if($allowedAccessProfileIds){
                $filterAllowedAccessProfile = $allowedAccessProfileIds;
            }
        }

        return $this->resourceRepository->getDataSetForEditProjectResource($project, $this->environment->getCurrentOrganizationId(), $filterNames, $filterOperator, $filterValue, $filterAllowedAccessProfile);
    }

    public function getAllowedResourcesToCompleateTheTask($taskId)
    {
        $isAllowed = false;
        $userProfiles = $this->environment->getCurrentUserProfiles();
        $allowedProfiles = $this->resourceRepository->getAllowedProfiles($taskId);
        if($allowedProfiles){
            foreach($allowedProfiles as $allowedProfile){
                if(in_array($allowedProfile["NRPERFILACESSO"], $userProfiles)){
                    $isAllowed = true;
                }
            }
        }else{
            $isAllowed = true;
        }
        return $isAllowed;


    }

    public function getAllResourceProjects($userId)
    {
        if ($userId == 0) {
            $userId = $this->environment->getCurrentUserId();
        }
        return $resourceProjectRepository = $this->resourceProjectRepository->getResourceProjects($userId);
    }

    public function getResourceIdByEmail($resourceEmail)
    {
        $resource = $this->resourceRepository
            ->findOneBy(array("email" => $resourceEmail, "status" => 'A'));
        return empty($resource) ? null : $resource->getId();
    }

    public function findAllManager(){
        $managers = $this->resourceRepository->getAllManagers($this->environment->getCurrentOrganizationId());
        return $managers;
    }

    public function getProjectManagerIdByEmail($resourceEmail)
    {
        $resource = $this->resourceRepository
            ->getProjectManagerIdByEmail($this->environment->getCurrentOrganizationId(), $resourceEmail);
        return empty($resource) ? null : $resource->getId();
    }

    public function getResourceIdByCNPJ($resourceCNPJ)
    {
        $resource = $this->resourceRepository
            ->findOneBy(array("documentNumber" => $resourceCNPJ, "status" => 'A', "class" => 'S'));
        return empty($resource) ? null : $resource->getId();
    }

    public function getNotProfileUsers($profile)
    {
        return $this->resourceRepository
            ->getUsersAgainstProfile($profile, $this->environment->getCurrentOrganizationId());
    }


    /**
     * @param \Zeedhi\DevOps\Model\Projectmanagement\Entities\Rule[] $profileRulesFinal
     * @param $ruleRepository
     * @param $partnerProfile
     * @return array
     */
    protected function prepareRules($profileRulesFinal, $ruleRepository, $partnerProfile)
    {

        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Rule[] $profileRules */
        $profileRules = $ruleRepository->findBy(array('profile' => $partnerProfile->getProfile()->getId()));
        if ($profileRules) {
            $arrayDiffRules = array();
            foreach ($profileRules as $profileRule) {
                foreach ($profileRulesFinal as $profileRuleFinal) {
                    if ($profileRule->getElement()->getId() == $profileRuleFinal->getElement()->getId()) {
                        if ($profileRule->getRuleType() != \Service\Acl::ALLOWED && $profileRuleFinal->getRuleType() != \Service\Acl::ALLOWED) {
                            if ($profileRule->getRuleType() == \Service\Acl::READ_ONLY) {
                                array_push($arrayDiffRules, $profileRule);
                            } else if ($profileRuleFinal->getRuleType() == \Service\Acl::READ_ONLY) {
                                array_push($arrayDiffRules, $profileRuleFinal);
                            } else if ($profileRule->getRuleType() == \Service\Acl::INVISIBLE) {
                                array_push($arrayDiffRules, $profileRule);
                            } else if ($profileRuleFinal->getRuleType() == \Service\Acl::INVISIBLE) {
                                array_push($arrayDiffRules, $profileRuleFinal);
                            }
                        }
                    }
                }
            }
            $profileRulesFinal = $arrayDiffRules;

        } else {
            $profileRulesFinal = array();
        }
        return $profileRulesFinal;
    }

    /**
     * Finds resource's structures from his id
     * @param int $resourceId
     * @param array $filteredStructures
     */
    public function findStructureByResourceId($resourceId)
    {
        $unfilteredStructures = $this->structureService->getStructuresFromManagerOnline();
        $filteredStructures = $unfilteredStructures;
        // We have not a tree grid, so we don't want to get the parent structure in the list
        /*$filteredStructures = array();
        if(!empty($resourceId)){ // if we have not a responsible, does not need return anything
            foreach($unfilteredStructures as &$structure){
                if(!empty($structure['PARENT'])){
                    $filteredStructures[] = $structure;
                }
            }
        }*/
        return $filteredStructures;
    }

    /**
     * @param $resourceId
     * @return array
     */
    public function findBasicCalendar($resourceId)
    {
        $calendars = array();
        /** @var Entities\Resource $resource */
        $resource = $this->resourceRepository->find($resourceId);
        if ($resource) {
            if ($resource->getCalendar()) {
                $calendars = $this->serviceCalendar->repositoryCalendar->getCalendar($resource->getCalendar()->getId());
                foreach ($calendars as &$calendar) {
                    $calendar['RESOURCE'] = $resourceId;
                }
            } else {
                $structure = $this->getStructureForResource($resource);
                if ($structure) {
                    $name = $this->environment->getNextEntityWithCalendar($structure)->getName();
                    $calendars = array(array(
                        'PARENT_CALENDAR_TEXT' => Messages::ALE_CALENDAR_IS_NOT_OWN . $name,
                        'CALENDAR_DEFAULT' => "",
                        'ID' => "",
                        'RESOURCE' => $resourceId)
                    );
                }
            }
        } 
        
        if (empty($calendars)){
            $calendars = array(array(
                'PARENT_CALENDAR_TEXT' => Messages::ALE_CALENDAR_IS_NOT_OWN,
                'CALENDAR_DEFAULT' => "",
                'ID' => "",
                'RESOURCE' => $resourceId)
            );
        }
        return $calendars;
    }
    
    public function getStructureForResource(Entities\Resource $resource){
        return $this->structureService->getStructureForResource($resource);
    }
    
    public function findExceptionCalendarDatesResource($resourceId, $beginDate, $endDate){

        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource */
        $resource = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Resource')->find($resourceId);
        $structure = $this->getStructureForResource($resource);
        return $this->serviceCalendar->findExceptionCalendarDatesResource($resource,$structure,$beginDate, $endDate);
    }
    
    public function findCalendarExceptionByResource($resourceId,$id)
    {
        $calendarException = array();
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource */
        $resource = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Resource')->find($resourceId);
        $structure = $this->getStructureForResource($resource);
        if ($structure){
            $calendarException = $this->serviceCalendar->findCalendarExceptionByResource($resource,$structure,$id);
        }
        
        return $calendarException;
    }
    
    public function findCalendarWeeklyDays($resourceId,$id)
    {
        $calendarWeeklyDays = array();
        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource */
        $resource = $this->entityManager->getRepository(Utilities::ENTITY_DIR.'Resource')->find($resourceId);
        $structure = $this->getStructureForResource($resource);
        if ($structure){
            $calendarWeeklyDays = $this->serviceCalendar->findCalendarWeeklyDaysByResource($resource,$structure,$id);
        }
        
        return $calendarWeeklyDays;
    }
    
    public function saveEditCalendar($id,$calendarDefaultId,$structureId){
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource */
            $resource = $this->resourceRepository->find($structureId);
            $nameCalendar = $resource->getId().'_'.$resource->getName();
            $calendar = $this->serviceCalendar->saveEditCalendar($id,$nameCalendar,null,null,'A',Calendar::CALENDAR_RESOURCE);
            if($calendarDefaultId){
                $calendarDefault = $this->serviceCalendar->find($calendarDefaultId);
                $calendar->setCalendarDefault($calendarDefault);
                $resource->setCalendar($calendar);
                $this->entityManager->persist($calendar);
                $this->entityManager->persist($resource);
                $this->entityManager->flush();
            }else{
                $resource->setCalendar(null);
                $this->serviceCalendar->calendarDelete($calendar);
            }

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return $calendar;
    }
    public function saveWeeklyDays($id, $calendar, $name, $typeSunday, $dailyHoursSunday,$typeMonday,
                                   $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
                                   $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate){
        $calendarDetail = $this->serviceCalendar->saveWeeklyDays($id, $calendar, $name, $typeSunday, $dailyHoursSunday,$typeMonday,
            $dailyHoursMonday, $typeTuesday, $dailyHoursTuesday, $typeWednesday, $dailyHoursWednesday, $typeThursday,
            $dailyHoursThursday, $typeFriday, $dailyHoursFriday, $typeSaturday, $dailyHoursSaturday, $beginDate, $endDate);
        return $calendarDetail;
    }

    public function findManagerExternalResourceIdsByIdentityProfiles() {
        return $this->resourceRepository->findResourceManagerProfilesByOrganization($this->environment->getCurrentOrganizationId());
    }
    public function findResourceManagerProfilesByOrganization($filter) {

        $filter = Utilities::createQueryFiltered($filter);
        return $this->resourceRepository->findResourceManagerProfilesByOrganization($this->environment->getCurrentOrganizationId(),$filter);
    }

    public function isManager(Entities\Resource $manager)
    {
        $externalIds = DevOpsUtilities::arrayAssociativeToArray($this->findManagerExternalResourceIdsByIdentityProfiles());
        return in_array($manager->getId(), $externalIds);
    }

    /**
     * @param Entities\Resource[] $managers
     * @return bool
     */
    public function isManagers(array $managers)
    {
        $externalIds = DevOpsUtilities::arrayAssociativeToArray($this->findManagerExternalResourceIdsByIdentityProfiles());
        $validate = true ;
        foreach ($managers as $manager) {
            if(!in_array($manager->getId(), $externalIds)){
                $validate = false;
            }
        }
        return $validate;
    }
}
